function [x,y] = harris(fun,x,options,varargin)
%
% optimisation par la methode de Harris
%
% fun: la fonction a optimiser (renvoie aussi le gradient)
% x0: etat initial
%
% options(1) = visualisation
% options(2) = nb max iterations
% options(3) = erreur 
% options(4) = pas pour l'affichage

pas = 0.1 * ones(size(x));
a = 1.2;
b = 0.8;
c = 0.5;
ovf = 1e4 ;
unf = 1e-6;
err=[];
it = 0;
gain=1;

[yp,gp]=feval(fun,x,varargin{:});

if isempty(gp),
   gp=gradnum(fun,x,yp,varargin{:});
end;
   
xp=x;
x = xp - pas .* gp;

while (gain >= options(3)) & (it <= options(2))
  it=it+1;
  [y,g]=feval(fun,x,varargin{:});
  if isempty(g),
    g=gradnum(fun,x,y,varargin{:});
end;
  if options(1) >0,
    if (rem(it,options(4))==1)|(options(4)==1), 
      fprintf(1,'%5.0f %7.4f %6.4g \n',[it y gain]); 
    % g1=gradnum(fun,x,y,varargin{:}); disp([g g1]);pause
    end;
  end;
  if y > yp,    
    x = xp;
    g = gp; 
    pas = pas * c;
    x = x - pas .* g;
  else
    gain=.9*gain+.1*abs(yp-y);
    xp = x;
    test = (g .* gp) >= 0;
    pas = ((test * a) + ((~test) * b)) .* pas;
    pas = (pas<=ovf) .* pas + (pas>ovf) * ovf;
    pas = (pas>=unf) .* pas + (pas<unf) * unf;
    gp = g;
    x = x - pas .* g;
    yp = y;
  end;
end



%------------------------------------------------------------------------------
function g=gradnum(fun,x,f,varargin);

epsi=1e-9;
p=length(x);
g=zeros(size(x));
for i=1:p,
   dx=zeros(size(x));;
   dx(i)=dx(i)+epsi;
   g(i)=(feval(fun,x+dx,varargin{:})-f)/epsi;
end;

